@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -91,14 +91,14 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
             else:
               _AppendFirstLeafTokenSubtype(
                   child, format_token.Subtype.DICTIONARY_VALUE)
-          elif (child is not None and (isinstance(child, pytree.Node) or
-                                       (not child.value.startswith('#') and
-                                        child.value not in '{:,'))):
+          elif (child is not None and (isinstance(child, pytree.Node) or (
+              not child.value.startswith('#') and child.value not in '{:,'))):
             # Mark the first leaf of a key entry as a DICTIONARY_KEY. We
             # normally want to split before them if the dictionary cannot exist
             # on a single line.
             _AppendFirstLeafTokenSubtype(child,
                                          format_token.Subtype.DICTIONARY_KEY)
+            _AppendSubtypeRec(child, format_token.Subtype.DICTIONARY_KEY_PART)
         last_was_colon = pytree_utils.NodeName(child) == 'COLON'
 
   def Visit_expr_stmt(self, node):  # pylint: disable=invalid-name
@@ -243,6 +243,14 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
     self._ProcessArgLists(node)
     _SetDefaultOrNamedAssignArgListSubtype(node)
 
+  def Visit_decorator(self, node):  # pylint: disable=invalid-name
+    # decorator ::=
+    #     '@' dotted_name [ '(' [arglist] ')' ] NEWLINE
+    for child in node.children:
+      if isinstance(child, pytree.Leaf) and child.value == '@':
+        _AppendTokenSubtype(child, subtype=format_token.Subtype.DECORATOR)
+      self.Visit(child)
+
   def Visit_funcdef(self, node):  # pylint: disable=invalid-name
     # funcdef ::=
     #     'def' NAME parameters ['->' test] ':' suite
@@ -296,8 +304,10 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
 
 
 def _SetDefaultOrNamedAssignArgListSubtype(node):
+  """Set named assign subtype on elements in a arg list."""
 
   def HasDefaultOrNamedAssignSubtype(node):
+    """Return True if the arg list has a named assign subtype."""
     if isinstance(node, pytree.Leaf):
       if (format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in
           pytree_utils.GetNodeAnnotation(node, pytree_utils.Annotation.SUBTYPE,
@@ -306,7 +316,8 @@ def _SetDefaultOrNamedAssignArgListSubtype(node):
       return False
     has_subtype = False
     for child in node.children:
-      has_subtype |= HasDefaultOrNamedAssignSubtype(child)
+      if pytree_utils.NodeName(child) != 'arglist':
+        has_subtype |= HasDefaultOrNamedAssignSubtype(child)
     return has_subtype
 
   if HasDefaultOrNamedAssignSubtype(node):
